@@ -1,5 +1,5 @@
 ;;; gnus-ems.el --- functions for making Gnus work under different Emacsen
-;; Copyright (C) 1995,96,97,98 Free Software Foundation, Inc.
+;; Copyright (C) 1995,96,97,98,99 Free Software Foundation, Inc.
 
 ;; Author: Lars Magne Ingebrigtsen <larsi@gnus.org>
 ;; Keywords: news
@@ -33,7 +33,9 @@
   "Non-nil if running under XEmacs.")
 
 (defvar gnus-mouse-2 [mouse-2])
+(defvar gnus-down-mouse-3 [down-mouse-3])
 (defvar gnus-down-mouse-2 [down-mouse-2])
+(defvar gnus-widget-button-keymap nil)
 (defvar gnus-mode-line-modified
   (if (or gnus-xemacs
 	  (< emacs-major-version 20))
@@ -45,44 +47,15 @@
   (autoload 'gnus-xmas-redefine "gnus-xmas")
   (autoload 'appt-select-lowest-window "appt"))
 
-(or (fboundp 'mail-file-babyl-p)
-    (fset 'mail-file-babyl-p 'rmail-file-p))
-
 ;;; Mule functions.
 
-(defun gnus-mule-cite-add-face (number prefix face)
-  ;; At line NUMBER, ignore PREFIX and add FACE to the rest of the line.
-  (when face
-    (let ((inhibit-point-motion-hooks t)
-	  from to)
-      (goto-line number)
-      (unless (eobp)            ; Sometimes things become confused (broken).
-        (if (boundp 'MULE)
-            (forward-char (chars-in-string prefix))
-          (forward-char (length prefix)))
-        (skip-chars-forward " \t")
-        (setq from (point))
-        (end-of-line 1)
-        (skip-chars-backward " \t")
-        (setq to (point))
-        (when (< from to)
-          (push (setq overlay (gnus-make-overlay from to))
-                gnus-cite-overlay-list)
-          (gnus-overlay-put (gnus-make-overlay from to) 'face face))))))
-
 (defun gnus-mule-max-width-function (el max-width)
-  (` (let* ((val (eval (, el)))
-	    (valstr (if (numberp val)
-			(int-to-string val) val)))
-       (if (> (length valstr) (, max-width))
-	   (truncate-string valstr (, max-width))
-	 valstr))))
-
-(defun gnus-encode-coding-string (string system)
-  string)
-
-(defun gnus-decode-coding-string (string system)
-  string)
+  `(let* ((val (eval (, el)))
+	  (valstr (if (numberp val)
+		      (int-to-string val) val)))
+     (if (> (length valstr) ,max-width)
+	 (truncate-string-to-width valstr ,max-width)
+       valstr)))
 
 (eval-and-compile
   (if (string-match "XEmacs\\|Lucid" emacs-version)
@@ -95,20 +68,6 @@
    ((string-match "XEmacs\\|Lucid" emacs-version)
     (gnus-xmas-define))
 
-   ((or (not (boundp 'emacs-minor-version))
-	(and (< emacs-major-version 20)
-	     (< emacs-minor-version 30)))
-    ;; Remove the `intangible' prop.
-    (let ((props (and (boundp 'gnus-hidden-properties)
-		      gnus-hidden-properties)))
-      (while (and props (not (eq (car (cdr props)) 'intangible)))
-	(setq props (cdr props)))
-      (when props
-	(setcdr props (cdr (cdr (cdr props))))))
-    (unless (fboundp 'buffer-substring-no-properties)
-      (defun buffer-substring-no-properties (beg end)
-	(format "%s" (buffer-substring beg end)))))
-
    ((boundp 'MULE)
     (provide 'gnusutil))))
 
@@ -121,23 +80,18 @@
       (while funcs
 	(unless (fboundp (car funcs))
 	  (fset (car funcs) 'gnus-dummy-func))
-	(setq funcs (cdr funcs))))))
-  (unless (fboundp 'file-regular-p)
-    (defun file-regular-p (file)
-      (and (not (file-directory-p file))
-	   (not (file-symlink-p file))
-	   (file-exists-p file))))
-  (unless (fboundp 'face-list)
-    (defun face-list (&rest args))))
+	(setq funcs (cdr funcs)))))))
 
 (eval-and-compile
   (let ((case-fold-search t))
     (cond
-     ((string-match "windows-nt\\|os/2\\|emx" (format "%s" system-type))
+     ((string-match "windows-nt\\|os/2\\|emx\\|cygwin32"
+		    (symbol-name system-type))
       (setq nnheader-file-name-translation-alist
 	    (append nnheader-file-name-translation-alist
-		    '((?: . ?_)
-		      (?+ . ?-))))))))
+		    (mapcar (lambda (c) (cons c ?_))
+			    '(?: ?* ?\" ?< ?> ??))
+		    '((?+ . ?-))))))))
 
 (defvar gnus-tmp-unread)
 (defvar gnus-tmp-replied)
@@ -159,7 +113,7 @@
 
     ;; [Note] Now there are three kinds of mule implementations,
     ;; original MULE, XEmacs/mule and beta version of Emacs including
-    ;; some mule features. Unfortunately these API are different. In
+    ;; some mule features.  Unfortunately these API are different.  In
     ;; particular, Emacs (including original MULE) and XEmacs are
     ;; quite different.
     ;; Predicates to check are following:
@@ -168,21 +122,16 @@
     ;; (featurep 'mule) is t when every mule variants are running.
 
     ;; These implementations may be able to share between original
-    ;; MULE and beta version of new Emacs. In addition, it is able to
+    ;; MULE and beta version of new Emacs.  In addition, it is able to
     ;; detect XEmacs/mule by (featurep 'mule) and to check variable
-    ;; `emacs-version'. In this case, implementation for XEmacs/mule
+    ;; `emacs-version'.  In this case, implementation for XEmacs/mule
     ;; may be able to share between XEmacs and XEmacs/mule.
 
-    (defalias 'gnus-truncate-string 'truncate-string)
-
     (defvar gnus-summary-display-table nil
       "Display table used in summary mode buffers.")
-    (fset 'gnus-cite-add-face 'gnus-mule-cite-add-face)
     (fset 'gnus-max-width-function 'gnus-mule-max-width-function)
     (fset 'gnus-summary-set-display-table (lambda ()))
-    (fset 'gnus-encode-coding-string 'encode-coding-string)
-    (fset 'gnus-decode-coding-string 'decode-coding-string)
-    
+
     (when (boundp 'gnus-check-before-posting)
       (setq gnus-check-before-posting
 	    (delq 'long-lines
@@ -199,13 +148,12 @@
 	  (format "%4d: %-20s"
 		  gnus-tmp-lines
 		  (if (> (length gnus-tmp-name) 20)
-		      (truncate-string gnus-tmp-name 20)
+		      (truncate-string-to-width gnus-tmp-name 20)
 		    gnus-tmp-name))
 	  gnus-tmp-closing-bracket)
 	 (point))
        gnus-mouse-face-prop gnus-mouse-face)
-      (insert " " gnus-tmp-subject-or-nil "\n"))
-    )))
+      (insert " " gnus-tmp-subject-or-nil "\n")))))
 
 (defun gnus-region-active-p ()
   "Say whether the region is active."
@@ -217,6 +165,7 @@
 (defun gnus-add-minor-mode (mode name map)
   (if (fboundp 'add-minor-mode)
       (add-minor-mode mode name map)
+    (set (make-local-variable mode) t)
     (unless (assq mode minor-mode-alist)
       (push `(,mode ,name) minor-mode-alist))
     (unless (assq mode minor-mode-map-alist)
@@ -228,28 +177,27 @@
   (let ((dir (nnheader-find-etc-directory "gnus"))
 	pixmap file height beg i)
     (save-excursion
-      (switch-to-buffer (get-buffer-create gnus-group-buffer))
+      (switch-to-buffer (gnus-get-buffer-create gnus-group-buffer))
       (let ((buffer-read-only nil))
 	(erase-buffer)
 	(when (and dir
 		   (file-exists-p (setq file (concat dir "x-splash"))))
-	  (nnheader-temp-write nil
+	  (with-temp-buffer
 	    (insert-file-contents file)
 	    (goto-char (point-min))
 	    (ignore-errors
 	      (setq pixmap (read (current-buffer))))))
 	(when pixmap
-	  (erase-buffer)
 	  (unless (facep 'gnus-splash)
 	    (make-face 'gnus-splash))
 	  (setq height (/ (car pixmap) (frame-char-height))
 		width (/ (cadr pixmap) (frame-char-width)))
-	  (set-face-foreground 'gnus-splash "ForestGreen")
+	  (set-face-foreground 'gnus-splash "Brown")
 	  (set-face-stipple 'gnus-splash pixmap)
 	  (insert-char ?\n (* (/ (window-height) 2 height) height))
 	  (setq i height)
 	  (while (> i 0)
-	    (insert-char ?  (* (+ (/ (window-width) 2 width) 1) width))
+	    (insert-char ?  (* (/ (window-width) 2 width) width))
 	    (setq beg (point))
 	    (insert-char ?  width)
 	    (set-text-properties beg (point) '(face gnus-splash))
