@@ -1,5 +1,5 @@
 ;;; gnus-win.el --- window configuration functions for Gnus
-;; Copyright (C) 1996,97,98 Free Software Foundation, Inc.
+;; Copyright (C) 1996,97,98,99 Free Software Foundation, Inc.
 
 ;; Author: Lars Magne Ingebrigtsen <larsi@gnus.org>
 ;; Keywords: news
@@ -85,9 +85,9 @@
 		  (article 1.0)))
       (t
        '(vertical 1.0
-		 (summary 0.25 point)
-		 (if gnus-carpal '(summary-carpal 4))
-		 (article 1.0)))))
+		  (summary 0.25 point)
+		  (if gnus-carpal '(summary-carpal 4))
+		  (article 1.0)))))
     (server
      (vertical 1.0
 	       (server 1.0 point)
@@ -154,6 +154,10 @@
      (vertical 1.0
 	       (summary 0.5 point)
 	       ("*Score Words*" 1.0)))
+    (split-trace
+     (vertical 1.0
+	       (summary 0.5 point)
+	       ("*Split Trace*" 1.0)))
     (category
      (vertical 1.0
 	       (category 1.0)))
@@ -185,6 +189,7 @@ See the Gnus manual for an explanation of the syntax used.")
     (picons . gnus-picons-buffer-name)
     (tree . gnus-tree-buffer)
     (score-trace . "*Score Trace*")
+    (split-trace . "*Split Trace*")
     (info . gnus-info-buffer)
     (category . gnus-category-buffer)
     (article-copy . gnus-article-copy)
@@ -281,7 +286,7 @@ See the Gnus manual for an explanation of the syntax used.")
 (defun gnus-configure-frame (split &optional window)
   "Split WINDOW according to SPLIT."
   (unless window
-    (setq window (get-buffer-window (current-buffer))))
+    (setq window (or (get-buffer-window (current-buffer)) (selected-window))))
   (select-window window)
   ;; This might be an old-stylee buffer config.
   (when (vectorp split)
@@ -313,9 +318,11 @@ See the Gnus manual for an explanation of the syntax used.")
       (let ((buffer (cond ((stringp type) type)
 			  (t (cdr (assq type gnus-window-to-buffer))))))
 	(unless buffer
-	  (error "Illegal buffer type: %s" type))
-	(switch-to-buffer (get-buffer-create
-			   (gnus-window-to-buffer-helper buffer)))
+	  (error "Invalid buffer type: %s" type))
+	(let ((buf (gnus-get-buffer-create
+ 		    (gnus-window-to-buffer-helper buffer))))
+ 	  (if (eq buf (window-buffer (selected-window))) (set-buffer buf)
+ 	    (switch-to-buffer buf)))
 	(when (memq 'frame-focus split)
 	  (setq gnus-window-frame-focus window))
 	;; We return the window if it has the `point' spec.
@@ -368,7 +375,7 @@ See the Gnus manual for an explanation of the syntax used.")
 		  ((integerp size)
 		   (setq s size))
 		  (t
-		   (error "Illegal size: %s" size)))
+		   (error "Invalid size: %s" size)))
 	    ;; Try to make sure that we are inside the safe limits.
 	    (cond ((zerop s))
 		  ((eq type 'horizontal)
@@ -403,58 +410,54 @@ See the Gnus manual for an explanation of the syntax used.")
 (defvar gnus-frame-split-p nil)
 
 (defun gnus-configure-windows (setting &optional force)
-  (setq gnus-current-window-configuration setting)
-  (setq force (or force gnus-always-force-window-configuration))
-  (setq setting (gnus-windows-old-to-new setting))
-  (let ((split (if (symbolp setting)
-		   (cadr (assq setting gnus-buffer-configuration))
-		 setting))
-	all-visible)
+  (if (window-configuration-p setting)
+      (set-window-configuration setting)
+    (setq gnus-current-window-configuration setting)
+    (setq force (or force gnus-always-force-window-configuration))
+    (setq setting (gnus-windows-old-to-new setting))
+    (let ((split (if (symbolp setting)
+		     (cadr (assq setting gnus-buffer-configuration))
+		   setting))
+	  all-visible)
 
-    (setq gnus-frame-split-p nil)
+      (setq gnus-frame-split-p nil)
 
-    (unless split
-      (error "No such setting: %s" setting))
+      (unless split
+	(error "No such setting in `gnus-buffer-configuration': %s" setting))
 
-    (if (and (setq all-visible (gnus-all-windows-visible-p split))
-	     (not force))
-	;; All the windows mentioned are already visible, so we just
-	;; put point in the assigned buffer, and do not touch the
-	;; winconf.
-	(select-window all-visible)
+      (if (and (setq all-visible (gnus-all-windows-visible-p split))
+	       (not force))
+	  ;; All the windows mentioned are already visible, so we just
+	  ;; put point in the assigned buffer, and do not touch the
+	  ;; winconf.
+	  (select-window all-visible)
 
-      ;; Either remove all windows or just remove all Gnus windows.
-      (let ((frame (selected-frame)))
-	(unwind-protect
-	    (if gnus-use-full-window
-		;; We want to remove all other windows.
-		(if (not gnus-frame-split-p)
-		    ;; This is not a `frame' split, so we ignore the
-		    ;; other frames.
-		    (delete-other-windows)
-		  ;; This is a `frame' split, so we delete all windows
-		  ;; on all frames.
-		  (gnus-delete-windows-in-gnusey-frames))
-	      ;; Just remove some windows.
-	      (gnus-remove-some-windows)
-	      (switch-to-buffer nntp-server-buffer))
-	  (select-frame frame)))
+	;; Either remove all windows or just remove all Gnus windows.
+	(let ((frame (selected-frame)))
+	  (unwind-protect
+	      (if gnus-use-full-window
+		  ;; We want to remove all other windows.
+		  (if (not gnus-frame-split-p)
+		      ;; This is not a `frame' split, so we ignore the
+		      ;; other frames.
+		      (delete-other-windows)
+		    ;; This is a `frame' split, so we delete all windows
+		    ;; on all frames.
+		    (gnus-delete-windows-in-gnusey-frames))
+		;; Just remove some windows.
+		(gnus-remove-some-windows)
+	      (set-buffer nntp-server-buffer))
+	    (select-frame frame)))
 
-      (switch-to-buffer nntp-server-buffer)
-      (let (gnus-window-frame-focus)
-	(gnus-configure-frame split (get-buffer-window (current-buffer)))
-	(when gnus-window-frame-focus
-	  (select-frame (window-frame gnus-window-frame-focus)))))))
+	(let (gnus-window-frame-focus)
+	(set-buffer nntp-server-buffer)
+	(gnus-configure-frame split)
+	  (when gnus-window-frame-focus
+	    (select-frame (window-frame gnus-window-frame-focus))))))))
 
 (defun gnus-delete-windows-in-gnusey-frames ()
   "Do a `delete-other-windows' in all frames that have Gnus windows."
-  (let ((buffers
-	 (mapcar
-	  (lambda (elem)
-	    (let ((buf (gnus-window-to-buffer-helper (cdr elem))))
-	      (if (not (null buf))
-		  (get-buffer buf))))
-	  gnus-window-to-buffer)))
+  (let ((buffers (gnus-buffers)))
     (mapcar
      (lambda (frame)
        (unless (eq (cdr (assq 'minibuffer
@@ -501,11 +504,11 @@ should have point."
 	(setq buffer (cond ((stringp type) type)
 			   (t (cdr (assq type gnus-window-to-buffer)))))
 	(unless buffer
-	  (error "Illegal buffer type: %s" type))
+	  (error "Invalid buffer type: %s" type))
 	(if (and (setq buf (get-buffer (gnus-window-to-buffer-helper buffer)))
 		 (setq win (get-buffer-window buf t)))
 	    (if (memq 'point split)
-	      (setq all-visible win))
+		(setq all-visible win))
 	  (setq all-visible nil)))
        (t
 	(when (eq type 'frame)
@@ -518,39 +521,22 @@ should have point."
   (nth 1 (window-edges window)))
 
 (defun gnus-remove-some-windows ()
-  (let ((buffers gnus-window-to-buffer)
+  (let ((buffers (gnus-buffers))
 	buf bufs lowest-buf lowest)
     (save-excursion
       ;; Remove windows on all known Gnus buffers.
-      (while buffers
-	(and (setq buf (gnus-window-to-buffer-helper (cdar buffers)))
-	     (get-buffer-window buf)
-	     (progn
-	       (push buf bufs)
-	       (pop-to-buffer buf)
-	       (when (or (not lowest)
-			 (< (gnus-window-top-edge) lowest))
-		 (setq lowest (gnus-window-top-edge))
-		 (setq lowest-buf buf))))
-	(setq buffers (cdr buffers)))
-      ;; Remove windows on *all* summary buffers.
-      (walk-windows
-       (lambda (win)
-	 (let ((buf (window-buffer win)))
-	   (when (string-match	"^\\*\\(Dead \\)?Summary" (buffer-name buf))
-	     (push buf bufs)
-	     (pop-to-buffer buf)
-	     (when (or (not lowest)
-		       (< (gnus-window-top-edge) lowest))
-	       (setq lowest-buf buf)
-	       (setq lowest (gnus-window-top-edge)))))))
+      (while (setq buf (pop buffers))
+	(when (get-buffer-window buf)
+	  (push buf bufs)
+	  (pop-to-buffer buf)
+	  (when (or (not lowest)
+		    (< (gnus-window-top-edge) lowest))
+	    (setq lowest (gnus-window-top-edge)
+		  lowest-buf buf))))
       (when lowest-buf
 	(pop-to-buffer lowest-buf)
 	(switch-to-buffer nntp-server-buffer))
-      (while bufs
-	(when (not (eq (car bufs) lowest-buf))
-	  (delete-windows-on (car bufs)))
-	(setq bufs (cdr bufs))))))
+      (mapcar (lambda (b) (delete-windows-on b t)) bufs))))
 
 (provide 'gnus-win)
 
